package com.citygis.cdc.ServiceImpl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.citygis.cdc.pojo.Log;
import com.citygis.cdc.service.DataByUrlService;
import com.citygis.cdc.service.LogService;
import com.citygis.cdc.utils.SSLUtils;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.BufferedHttpEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;

/**
 * @author zyl
 * @date 2022-09-14 14:21
 */
@ConfigurationProperties("urls")
@Service
public class DataByUrlServiceImpl implements DataByUrlService {

    @Resource
    LogService logService;

    public String urlh;

    public String ip;

    public String getIp() {
        return ip;
    }

    public void setIp(String ip) {
        this.ip = ip;
    }

    public String getUrlh() {
        return urlh;
    }

    public void setUrlh(String urlh) {
        this.urlh = urlh;
    }

    @Override
    public String dataByUrl(String data, String url, HttpServletRequest request) throws UnsupportedEncodingException {
        if (url == null) {
            url = urlh;
        }

//        url = UrlUtil.encode(url, "utf-8");

        long responseTime = 0;
        try {
            System.out.println("转发路径url--->" + url);
            System.out.println("转发参数data--->" + data);
            // 获得Http客户端(可以理解为:你得先有一个浏览器;注意:实际上HttpClient与浏览器是不一样的)
            CloseableHttpClient httpClient = SSLUtils.createSSLClientDefault();
            // 创建Post请求
            HttpPost httpPost = new HttpPost(url);

            long startTime = System.currentTimeMillis();

            String s = null;

            if (data != null && data.length() > 0) {
                StringEntity entity = new StringEntity(data, "UTF-8");
                httpPost.setEntity(entity);
            } else {
                data = "无参数";
            }

            Enumeration headerNames = request.getHeaderNames();
            while (headerNames.hasMoreElements()) {
                String key = (String) headerNames.nextElement();
                String value = request.getHeader(key);
                if (!"content-length".equals(key)) {
                    httpPost.setHeader(key, value);
                }
            }
            // 响应模型
            CloseableHttpResponse response = null;
            try {
                // 由客户端执行(发送)Post请求
                response = httpClient.execute(httpPost);

                long endTime = System.currentTimeMillis();
                responseTime = endTime - startTime;

                // 从响应模型中获取响应实体
                HttpEntity responseEntity = response.getEntity();
                responseEntity = new BufferedHttpEntity(responseEntity);

                System.out.println("响应状态为:" + response.getStatusLine());
                if (responseEntity != null) {
                    System.out.println("响应内容长度为:" + responseEntity.getContentLength());
                    s = EntityUtils.toString(responseEntity, "UTF-8");
                    System.out.println(s);
                }
            } catch (Exception e) {

                this.addLog(url, data, e.getMessage(), "400", responseTime);

                return "数据源异常";
            } finally {
                try {
                    // 释放资源
                    if (httpClient != null) {
                        httpClient.close();
                    }
                    if (response != null) {
                        response.close();
                    }
                } catch (IOException e) {
                    this.addLog(url, data, e.getMessage(), "400", responseTime);
                }

                try {
                    this.addLog(url, data, s, response.getStatusLine().toString(), responseTime);
                } catch (Exception e) {
                    this.addLog(url, data, e.getMessage(), "400", responseTime);
                } finally {
                    return s;
                }
            }
        } catch (Exception e) {

            this.addLog(url, data, e.getMessage(), "400", responseTime);

            return "数据源异常";
        }
    }

    @Override
    public String dataByUrlGet(String data, String url, HttpServletRequest request) throws UnsupportedEncodingException {
        if (url == null) {
            url = urlh;
        }

//        url = UrlUtil.encode(url, "utf-8");

        List<NameValuePair> params = new ArrayList<>();
        if (data != null && data.length() > 0) {
            JSONObject jsonObject = JSON.parseObject(data);

            for (String key : jsonObject.keySet()) {
                String value = jsonObject.get(key).toString();
                params.add(new BasicNameValuePair(key, value));
            }
        } else {
            data = "无参数";
        }

        long responseTime = 0;

        try {
            System.out.println("转发路径url--->" + url);
            System.out.println("转发参数data--->" + data);

            // 获得Http客户端(可以理解为:你得先有一个浏览器;注意:实际上HttpClient与浏览器是不一样的)
            URI uri = new URIBuilder(url)
                    .addParameters(params)
                    .build();

            CloseableHttpClient httpClient = SSLUtils.createSSLClientDefault();
            // 创建Get请求
            HttpGet httpGet = new HttpGet(uri);

            long startTime = System.currentTimeMillis();

            String s = null;

            Enumeration headerNames = request.getHeaderNames();
            while (headerNames.hasMoreElements()) {
                String key = (String) headerNames.nextElement();
                String value = request.getHeader(key);
                if (!"content-length".equals(key)) {
                    httpGet.setHeader(key, value);
                }
            }
            // 响应模型
            CloseableHttpResponse response = null;
            try {
                // 由客户端执行(发送)Post请求
                response = httpClient.execute(httpGet);

                long endTime = System.currentTimeMillis();
                responseTime = endTime - startTime;

                // 从响应模型中获取响应实体
                HttpEntity responseEntity = response.getEntity();
                responseEntity = new BufferedHttpEntity(responseEntity);

                System.out.println("响应状态为:" + response.getStatusLine());
                if (responseEntity != null) {
                    System.out.println("响应内容长度为:" + responseEntity.getContentLength());
                    s = EntityUtils.toString(responseEntity, "UTF-8");
                    System.out.println(s);
                }
            } catch (Exception e) {

                this.addLog(url, data, e.getMessage(), "400", responseTime);

                return "数据源异常";
            } finally {
                try {
                    // 释放资源
                    if (httpClient != null) {
                        httpClient.close();
                    }
                    if (response != null) {
                        response.close();
                    }
                } catch (IOException e) {
                    this.addLog(url, data, e.getMessage(), "400", responseTime);
                }

                try {
                    this.addLog(url, data, s, response.getStatusLine().toString(), responseTime);
                } catch (Exception e) {
                    this.addLog(url, data, e.getMessage(), "400", responseTime);
                } finally {
                    return s;
                }
            }
        } catch (Exception e) {

            this.addLog(url, data, e.getMessage(), "400", responseTime);

            return "数据源异常";
        }
    }

    @Override
    public String dataByGMUrl(String data, String url, HttpServletRequest request) throws UnsupportedEncodingException {

        if (url == null) {
            url = urlh;
        }

//        url = UrlUtil.encode(url, "utf-8");

        long responseTime = 0;
        try {
            System.out.println("转发路径url--->" + url);
            System.out.println("转发参数data--->" + data);
            // 获得Http客户端(可以理解为:你得先有一个浏览器;注意:实际上HttpClient与浏览器是不一样的)
            CloseableHttpClient httpClient = SSLUtils.initGMSSL();
            // 创建Post请求
            HttpPost httpPost = new HttpPost(url);

            long startTime = System.currentTimeMillis();

            String s = null;

            if (data != null && data.length() > 0) {
                StringEntity entity = new StringEntity(data, "UTF-8");
                httpPost.setEntity(entity);
            } else {
                data = "无参数";
            }
            Enumeration headerNames = request.getHeaderNames();
            while (headerNames.hasMoreElements()) {
                String key = (String) headerNames.nextElement();
                String value = request.getHeader(key);
                if (!"content-length".equals(key)) {
                    httpPost.setHeader(key, value);
                }
            }
            // 响应模型
            CloseableHttpResponse response = null;
            try {
                // 由客户端执行(发送)Post请求
                response = httpClient.execute(httpPost);

                long endTime = System.currentTimeMillis();
                responseTime = endTime - startTime;

                // 从响应模型中获取响应实体
                HttpEntity responseEntity = response.getEntity();
                responseEntity = new BufferedHttpEntity(responseEntity);

                System.out.println("响应状态为:" + response.getStatusLine());
                if (responseEntity != null) {
                    System.out.println("响应内容长度为:" + responseEntity.getContentLength());
                    s = EntityUtils.toString(responseEntity, "UTF-8");
                    System.out.println(s);
                }
            } catch (Exception e) {

                this.addLog(url, data, e.getMessage(), "400", responseTime);

                return "数据源异常";
            } finally {
                try {
                    // 释放资源
                    if (httpClient != null) {
                        httpClient.close();
                    }
                    if (response != null) {
                        response.close();
                    }
                } catch (IOException e) {
                    this.addLog(url, data, e.getMessage(), "400", responseTime);
                }

                try {
                    this.addLog(url, data, s, response.getStatusLine().toString(), responseTime);
                } catch (Exception e) {
                    this.addLog(url, data, e.getMessage(), "400", responseTime);
                } finally {
                    return s;
                }
            }
        } catch (Exception e) {

            this.addLog(url, data, e.getMessage(), "400", responseTime);

            return "数据源异常";
        }
    }

    @Override
    public String dataByGMUrlGet(String data, String url, HttpServletRequest request) throws UnsupportedEncodingException {
        if (url == null) {
            url = urlh;
        }

//        url = UrlUtil.encode(url, "utf-8");

        List<NameValuePair> params = new ArrayList<>();
        if (data != null && data.length() > 0) {
            JSONObject jsonObject = JSON.parseObject(data);

            for (String key : jsonObject.keySet()) {
                String value = jsonObject.get(key).toString();
                params.add(new BasicNameValuePair(key, value));
            }
        } else {
            data = "无参数";
        }

        long responseTime = 0;

        try {
            System.out.println("转发路径url--->" + url);
            System.out.println("转发参数data--->" + data);

            // 获得Http客户端(可以理解为:你得先有一个浏览器;注意:实际上HttpClient与浏览器是不一样的)
            URI uri = new URIBuilder(url)
                    .addParameters(params)
                    .build();

            CloseableHttpClient httpClient = SSLUtils.initGMSSL();

            // 创建Get请求
            HttpGet httpGet = new HttpGet(uri);

            long startTime = System.currentTimeMillis();

            String s = null;

            Enumeration headerNames = request.getHeaderNames();
            while (headerNames.hasMoreElements()) {
                String key = (String) headerNames.nextElement();
                String value = request.getHeader(key);
                if (!"content-length".equals(key)) {
                    httpGet.setHeader(key, value);
                }
            }
            // 响应模型
            CloseableHttpResponse response = null;
            try {
                // 由客户端执行(发送)Post请求
                response = httpClient.execute(httpGet);

                long endTime = System.currentTimeMillis();
                responseTime = endTime - startTime;

                // 从响应模型中获取响应实体
                HttpEntity responseEntity = response.getEntity();
                responseEntity = new BufferedHttpEntity(responseEntity);

                System.out.println("响应状态为:" + response.getStatusLine());
                if (responseEntity != null) {
                    System.out.println("响应内容长度为:" + responseEntity.getContentLength());
                    s = EntityUtils.toString(responseEntity, "UTF-8");
                    System.out.println(s);
                }
            } catch (Exception e) {

                this.addLog(url, data, e.getMessage(), "400", responseTime);

                return "数据源异常";
            } finally {
                try {
                    // 释放资源
                    if (httpClient != null) {
                        httpClient.close();
                    }
                    if (response != null) {
                        response.close();
                    }
                } catch (IOException e) {
                    this.addLog(url, data, e.getMessage(), "400", responseTime);
                }

                try {
                    this.addLog(url, data, s, response.getStatusLine().toString(), responseTime);
                } catch (Exception e) {
                    this.addLog(url, data, e.getMessage(), "400", responseTime);
                } finally {
                    return s;
                }
            }
        } catch (Exception e) {

            this.addLog(url, data, e.getMessage(), "400", responseTime);

            return "数据源异常";
        }
    }

    public void addLog(String url, String data, String message, String status, Long responseTime) {
        Log log = new Log();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String format = sdf.format(new Date());
        log.setUrl(url);
        log.setPara(data.toString());
        log.setTime(format);
        log.setReponse(message);
        log.setStatus(status);
        log.setResponseTime(String.valueOf(responseTime));
        log.setIp(ip);
        logService.insertLog(log);
    }
}
